@@ -97,7 +97,15 @@ my_error (int errcode)
   return gpg_err_make (default_errsource, errcode);
 }
 
-
+/*
+ * get_max_fds, close_all_fds and get_all_open_fds are functions for
+ * POSIX, not Windows.  After fork and before exec on POSIX, those are
+ * used when spawning a child process so that the child process
+ * doesn't keep having file descriptors not needed for it.  On
+ * Windows, spawn API has a different semantics (than fork + exec).
+ */
+#undef EXPORT_UNUSED_FUNCTIONS
+#ifdef EXPORT_UNUSED_FUNCTIONS
 /* Return the maximum number of currently allowed open file
    descriptors.  Only useful on POSIX systems but returns a value on
    other systems too.  */
@@ -178,7 +186,7 @@ get_all_open_fds (void)
 #endif /*HAVE_STAT*/
   return array;
 }
-
+#endif
 
 /* Helper function to build_w32_commandline. */
 static char *
@@ -544,11 +552,14 @@ gnupg_spawn_process (const char *pgmname, const char *argv[],
     return err;
 
   if (inpipe[0] == INVALID_HANDLE_VALUE)
-    nullhd[0] = w32_open_null (0);
+    nullhd[0] = ((flags & GNUPG_SPAWN_KEEP_STDIN)?
+                 GetStdHandle (STD_INPUT_HANDLE) : w32_open_null (0));
   if (outpipe[1] == INVALID_HANDLE_VALUE)
-    nullhd[1] = w32_open_null (1);
+    nullhd[1] = ((flags & GNUPG_SPAWN_KEEP_STDOUT)?
+                 GetStdHandle (STD_OUTPUT_HANDLE) : w32_open_null (1));
   if (errpipe[1] == INVALID_HANDLE_VALUE)
-    nullhd[2] = w32_open_null (1);
+    nullhd[2] = ((flags & GNUPG_SPAWN_KEEP_STDERR)?
+                 GetStdHandle (STD_ERROR_HANDLE) : w32_open_null (1));
 
   /* Start the process.  Note that we can't run the PREEXEC function
      because this might change our own environment. */
