@@ -549,12 +549,14 @@ get_override_session_key (DEK *dek, const char *string)
 
 /* Stub: */
 int
-decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek)
+decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
+              int *compliance_error)
 {
   (void)ctrl;
   (void)procctx;
   (void)ed;
   (void)dek;
+  (void)compliance_error;
   return GPG_ERR_GENERAL;
 }
 
@@ -704,12 +706,12 @@ dotlock_remove_lockfiles (void)
 {
 }
 
-gpg_error_t
+int
 agent_probe_secret_key (ctrl_t ctrl, PKT_public_key *pk)
 {
   (void)ctrl;
   (void)pk;
-  return gpg_error (GPG_ERR_NO_SECKEY);
+  return 0;
 }
 
 gpg_error_t
